package ocaml_intrinsics

  1. Overview
  2. Docs

Module Native_pointer.ExpertSource

Sourceval of_nativeint : nativeint -> t

These are *not* equivalent to unsafe_of/to_value: they convert between a boxed nativeint and a boxed native pointer (as in Obj.magic), which is always legal because they have the same representation.

Sourceval to_nativeint : t -> nativeint