Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
ext_pointer.ml1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142(** [Ext_pointer] uses values of the OCaml type "int" to represent pointers to 2-byte aligned memory blocks allocated outside the OCaml heap. The least significant bit of the address of a 2-byte aligned memory block is 0. This bit is used by this library to represent the address as an OCaml int, which prevents the OCaml GC from following pointers to external memory. To encode an external pointer as int: set the least significant bit. To decode int into an external pointer: clear the least significant bit. Note that these encode and decode operations are not the same as tagging and untagging operations on OCaml representation of int, which involves shifting. [Ext_pointer] allows OCaml code to pass around and manipulate pointers to external memory blocks without using "naked pointers". *) type t = private int external create : int -> t = "%identity" (** [offset_by_2n_bytes t n] represents an external pointer to address [t + 2*n]. *) let offset_by_2n_bytes (t : t) n = (* Because the least significant bit of the pointer is implicitly zero, adding to the represented integer value adds twice as much to the represented pointer value. Argument is doubled to ensure that the resulting address is 2-byte aligned. Checking that the argument is even would result in worse code generation with a conditional branch. *) create ((t :> int) + n) ;; module Immediate (V : sig type t [@@immediate] end) = struct (** [load_immediate t] assumes without checking that the value pointed to by [t] is immediate. *) external unsafe_load_immediate : t -> V.t = "caml_ext_pointer_load_immediate" [@@noalloc] [@@builtin] [@@no_effects] (** [store_int t i] stores the immediate [i] to the memory address represented by [t]. *) external store_immediate : t -> V.t -> unit = "caml_ext_pointer_store_immediate" [@@noalloc] [@@builtin] [@@no_coeffects] end module Int = Immediate (Stdlib.Int) module Bool = Immediate (Stdlib.Bool) (** [load_int t] reads untagged int pointed to by [t] and returns the corresponding tagged int. This should only be used to read a value written by [store_untagged_int]. Otherwise, if the value has most significant bit set, it will be lost by tagging. To avoid it, use [load_unboxed_nativeint] and check before converting to int (should not allocate). The native C stub is the same for both. *) external load_untagged_int : t -> (int[@untagged]) = "caml_ext_pointer_load_untagged_int" "caml_ext_pointer_load_unboxed_nativeint" [@@noalloc] [@@builtin] [@@no_effects] (** [store_int t d] untags [d] and stores the result to the memory pointed to by [t]. *) external store_untagged_int : t -> (int[@untagged]) -> unit = "caml_ext_pointer_store_untagged_int" "caml_ext_pointer_store_unboxed_nativeint" [@@noalloc] [@@builtin] [@@no_coeffects] (** [load_unboxed_nativeint t] reads unboxed nativeint pointed to by [t] and returns the corresponding (boxed) nativeint allocated on the OCaml heap. *) external load_unboxed_nativeint : t -> (nativeint[@unboxed]) = "caml_ext_pointer_load_unboxed_nativeint_bytecode" "caml_ext_pointer_load_unboxed_nativeint" [@@noalloc] [@@builtin] [@@no_effects] (** [store_unboxed_nativeint t d] stores the unboxed nativeint to the memory pointed to by [t]. *) external store_unboxed_nativeint : t -> (nativeint[@unboxed]) -> unit = "caml_ext_pointer_store_unboxed_nativeint_bytecode" "caml_ext_pointer_store_unboxed_nativeint" [@@noalloc] [@@builtin] [@@no_coeffects] (** [load_unboxed_int64 t] reads unboxed int64 pointed to by [t] and returns the corresponding (boxed) int64 allocated on the OCaml heap. *) external load_unboxed_int64 : t -> (int64[@unboxed]) = "caml_ext_pointer_load_unboxed_int64_bytecode" "caml_ext_pointer_load_unboxed_int64" [@@noalloc] [@@builtin] [@@no_effects] (** [store_unboxed_int64 t d] stores the unboxed int64 to the memory pointed to by [t]. *) external store_unboxed_int64 : t -> (int64[@unboxed]) -> unit = "caml_ext_pointer_store_unboxed_int64_bytecode" "caml_ext_pointer_store_unboxed_int64" [@@noalloc] [@@builtin] [@@no_coeffects] (** [load_unboxed_int32 t] reads unboxed int32 pointed to by [t] and returns the corresponding (boxed) int32 allocated on the OCaml heap. *) external load_unboxed_int32 : t -> (int32[@unboxed]) = "caml_ext_pointer_load_unboxed_int32_bytecode" "caml_ext_pointer_load_unboxed_int32" [@@noalloc] [@@builtin] [@@no_effects] (** [store_unboxed_int32 t d] stores the unboxed int32 to the memory pointed to by [t]. *) external store_unboxed_int32 : t -> (int32[@unboxed]) -> unit = "caml_ext_pointer_store_unboxed_int32_bytecode" "caml_ext_pointer_store_unboxed_int32" [@@noalloc] [@@builtin] [@@no_coeffects] (** For float operations, the pointer must be aligned at least to the native integer machine width (meaning on 32-bit platforms, a 32-bit-aligned pointer is acceptable even though the width of the float is 64 bits). *) (** [load_unboxed_float t] reads the unboxed float pointed to by [t]. (If the result is not directly passed to another operation expecting an unboxed float, then it will be boxed.) *) external load_unboxed_float : t -> (float[@unboxed]) = "caml_ext_pointer_load_unboxed_float_bytecode" "caml_ext_pointer_load_unboxed_float" [@@noalloc] [@@builtin] [@@no_effects] (** [store_unboxed_float t d] stores the unboxed float to the memory pointed to by [t]. *) external store_unboxed_float : t -> (float[@unboxed]) -> unit = "caml_ext_pointer_store_unboxed_float_bytecode" "caml_ext_pointer_store_unboxed_float" [@@noalloc] [@@builtin] [@@no_coeffects]