Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Native_pointer.ExpertSourceThese are *not* equivalent to unsafe_of/to_value: they convert between a boxed nativeint and a boxed native pointer (as in Obj.magic), which is always legal because they have the same representation.